//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//% TEXT2CLP.C - Copy the contents of a named file to the Windows Clipboard %
//% Written by James Greene Informatics Consulting, Lrrach, Germany        %
//%-------------------------------------------------------------------------%
//% Last Modified:                                                          %
//%     20.01.1999 - Creation                                               %
//%     5.02.2003 - Fixed an off-by-one bug reported by						%
//%					Mikael Karlsson [mikael.karlsson@pitch.se]				%
//%-------------------------------------------------------------------------%
//% TEXT2CLP Version 1.0 (20. January 1999)                                 %
//% Copyright (c) 1999, 2003 by James Greene								%
//% TEXT2CLP comes with NO WARRANTY to the extent permitted by law.         %
//% You may redistribute copies of TEXT2CLP under the terms of the          %
//% GNU General Public License. For more information about these            %
//% matters, see the file named COPYING, or contact the author at           %
//% <greene@gucc.org>                                                       %
//%                                                                         %
//% Please report any bugs to <greene@gucc.org>                             %
//%                                                                         %
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <conio.h>
#include <errno.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <fcntl.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define VER_NUMBER  "1.1"
#define VER_STRING  "Version " VER_NUMBER
#define VER_DATE    "5. February 2003"

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

typedef struct tagOPTIONS {
    int fQuiet: 1;
    int fTranslate: 1;
    int fDisplay: 1;
    int fHelp: 1;
    int fVersion: 1;
    char szName[MAX_PATH];
} OPTIONS;

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static  void    AnyKey(void) {
    register int ri;
    char *szMsg;

    szMsg = "(press any key to continue)";
    
    fprintf(stderr, szMsg);
    fflush(stderr);
    
    if (!getch())
        getch();
        
    for (ri=strlen(szMsg) ; ri > 0; ri--) 
        fprintf(stderr, "\b \b");
    fflush(stderr);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static void Version(void) {
    fprintf(stderr, "TEXT2CLP Version %s (%s at %.5s)\n", VER_NUMBER, VER_DATE, __TIME__);
    fprintf(stderr, "Copyright (c) 1999, 2003 by James Greene\n");
    fprintf(stderr, "TEXT2CLP comes with NO WARRANTY to the extent permitted by law.\n");
    fprintf(stderr, "You may redistribute copies of TEXT2CLP under the terms of the\n");
    fprintf(stderr, "GNU General Public License. For more information about these\n");
    fprintf(stderr, "matters, see the file named COPYING.txt, or contact the author\n");
    fprintf(stderr, "at <greene@informatics-consulting.de>\n");
    fprintf(stderr, "\nPlease report any bugs to <greene@informatics-consulting.de>\n\n");
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static  void    Usage(char *exe) {
    fprintf(stderr, "Usage: %s [options] filename [options]\n\n", exe);
    
    fprintf(stderr, "Options:  ( /option or --option are also accepted ) \n");
    fprintf(stderr, "  -d[isplay]   = Display the clipboard contents on the screen.\n");
    fprintf(stderr, "  -h[elp]      = Display this usage information.\n");
    fprintf(stderr, "  -n[ologo]    = Don't display the Copyright banner.\n");
    fprintf(stderr, "  -q[uiet]     = Same as -nologo.\n");
    fprintf(stderr, "  -s[ilent]    = Same as -nologo.\n");
    fprintf(stderr, "  -t[ranslate] = Convert text from MS-DOS to Windows ANSI to codes.\n");
    fprintf(stderr, "  -?           = Same as -help.\n");
    fprintf(stderr, "  The following options are all interpreted the same:\n");
    fprintf(stderr, "     -d, /d, -DISP, /Disp, --DISPLAY, -dIsP, ... you get the idea!\n\n");

    AnyKey();
    
    fprintf(stderr, "Examples:\n");
    fprintf(stderr, "  %s FOO.TXT\n", exe);
    fprintf(stderr, "  Copies the contents of FOO.TXT to the Windows Clipboard.\n");
    fprintf(stderr, "  %s -t \"C:\\My Documents\\Long File Name.txt\"\n", exe);
    fprintf(stderr, "  Copies the contents of the specified file to the Clipboard as OEM text.\n");
    fprintf(stderr, "  Filenames containing spaces must be \"enclosed in double quotes.\"\n\n");

    Version();
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static  BOOL ParseCommandLine(char *argv[], OPTIONS *opt) {
    char *pgm, *arg;
    register int ri;
    char *MoreInfo;

    MoreInfo = "Try `TEXT2CLP --help' for more information.\n";

    memset(opt, 0, sizeof(OPTIONS));

    pgm = *argv;
    
    while (arg = *++argv) {
        if ((*arg == '/') || (*arg == '-')) {   // flag
            ri = strlen(++arg);
            if (*arg == '-')
                ri = strlen(++arg);
          
            if (_strnicmp(arg, "TRANSLATE", ri) == 0) 
                opt->fTranslate = TRUE;
            else if (_strnicmp(arg, "DISPLAY", ri) == 0)
                opt->fDisplay = TRUE;
            else if ((_strnicmp(arg, "NOLOGO", ri) == 0) ||
                      (_strnicmp(arg, "QUIET", ri) == 0) ||
                      (_strnicmp(arg, "SILENT", ri) == 0))
                opt->fQuiet = TRUE;
            else if ((_strnicmp(arg, "HELP", ri) == 0) ||
                      (_strnicmp(arg, "?", ri) == 0))
                opt->fHelp = TRUE;
            else if ((_strnicmp(arg, "VERSION", ri) == 0))
                opt->fVersion = TRUE;
            else
                fprintf(stderr, "TEXT2CLP: Option '%s' not recognised\n%s", --arg, MoreInfo);
        } else {
            if (!*(opt->szName))
                strcpy(opt->szName, arg);
            else
                fprintf(stderr, "TEXT2CLP: Too many parameters: '%s' ignored\n%s", arg, MoreInfo);
        }
    }

    if (!opt->fQuiet) {
        fprintf(stderr, "TEXT2CLP: Copy a text file to the Windows Clipboard         %14s\n", VER_STRING);
        fprintf(stderr, "Copyright (c) 1999, 2003 by James Greene    <greene@informatics-consulting.de>\n");
        fprintf(stderr, "                            http://www.informatics-consulting.de\n\n");
    }

    if (opt->fHelp) {
        Usage(pgm);
        return FALSE;
    }

    if (opt->fVersion) 
        Version();
        
    // if no filename specified, then just display the text to the screen
    if (!*(opt->szName)) {
        fprintf(stderr, "TEXT2CLP: No filename specified.\n%s", MoreInfo);
        return FALSE;
    }

    return TRUE;
}
        
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
static  BOOL    ClipCopy(OPTIONS *opt) {
    HANDLE nd;
    LPSTR ns;
    register int ri;
    int fh;
    struct _stat buf;
    DWORD dwSize, dwSeg;
    int nRest;
#define SEGMENT     16384

    if (!(*opt->szName)) {                          // this should never happen
        fprintf(stderr, "TEXT2CLP: No filename specified.\n");
        return FALSE;
    }
        
    if (_stat(opt->szName, &buf) != -1) {
        dwSize = buf.st_size;
    } else {
        fprintf(stderr, "TEXT2CLP: Can't read %s: %s\n", opt->szName, strerror(errno));
        return FALSE;
    }

// allocate some memory for the file    

	// 05. Feb. 2003 - Added +2 to the memory allocation - 
	//		without it, there was a risk of the loss of the final character from the file.

    if (nd = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE, dwSize+2)) {
        if (ns = (LPSTR)GlobalLock(nd)) {

// open the specified file

            if ((fh = _open(opt->szName, O_RDONLY | O_BINARY)) == -1) {
                fprintf(stderr, "TEXT2CLP: Unable to open file %s for reading: %s\n",
                        opt->szName, strerror(errno));
                *ns = '\0';
            } else {
                dwSeg = 0L;
                nRest = min(SEGMENT, dwSize);
                
                while (nRest) {
                    if ((ri = read(fh, ns, nRest)) < 0) {
                        fprintf(stderr, "TEXT2CLP: Unable to read file %s: %s\n",
                            opt->szName, strerror(errno));
                        break;
                    }

                    ns[ri] = '\0';
                            
                    if (opt->fDisplay)
                        fputs(ns, stdout);
                        
                    ns += ri;
                    dwSeg += ri;
                    nRest = min(nRest, (int)(dwSize - dwSeg));

                }
                
                close(fh);
            }

            GlobalUnlock(nd);

            if (OpenClipboard(NULL)) {
                EmptyClipboard();       
                SetClipboardData(opt->fTranslate ? CF_OEMTEXT : CF_TEXT, nd);
                CloseClipboard();
    
                if (!opt->fQuiet) 
                    fprintf(stderr, "TEXT2CLP: %ld bytes copied to the Windows clipboard.\n", dwSize);
            } else 
                fprintf(stderr, "TEXT2CLP: Unable to open the Windows clipboard!\n");
        }
    }
    else
        fprintf(stderr, "TEXT2CLP: Can't allocate %ld bytes memory\n", dwSize);

    return TRUE;
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
int main(int argc, char *argv[]) {
    OPTIONS opt;

    return ParseCommandLine(argv, &opt) && ClipCopy(&opt) ? 1 : 0;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

